using System;
using System.Text;

using Org.BouncyCastle.Crypto;
using Org.BouncyCastle.Crypto.Engines;
using Org.BouncyCastle.Crypto.Parameters;
using Org.BouncyCastle.Security;
using Org.BouncyCastle.Crypto.Paddings;
using Org.BouncyCastle.Utilities;
using Org.BouncyCastle.Utilities.Encoders;
using Org.BouncyCastle.Crypto.Utilities;
using Org.BouncyCastle.Crypto.Modes;
using Org.BouncyCastle.Utilities.IO;
using Org.BouncyCastle.Crypto.Fpe;

public class FpeCipherExamples
{
 
    public static void Main(String[] args)
    {
        
        Console.WriteLine();
        Console.WriteLine();
        Console.WriteLine("==============================");
        Console.WriteLine("Start FPE Cipher Examples.Main");
        Console.WriteLine("==============================");

        {
            //Example 18 – FF3-1 FPE Cipher
            Console.WriteLine();
            Console.WriteLine("*** Example 18 – FF3-1 FPE Cipher ***");
            ExValues.cSharpFixedRandom.SetSeed(151);
            KeyParameter keyParam = keyParameterGeneration(128);
            
            char[] cipherTextData =  ff3_1FPEEncrypt(keyParam, "0123456789".ToCharArray(),"9988556611223344557700".ToCharArray());
            Console.WriteLine(string.Format("Encrypt FF3-1 FPE Cipher\tplainTextData:[{0}]\tLength:[{1}]\tcipherTextData:[{2}]\tLength:[{3}]",
                "9988556611223344557700", "9988556611223344557700".Length,
                new string(cipherTextData), new string(cipherTextData).Length ));
            char[] plainTextData = ff3_1FPEDecrypt(keyParam,"0123456789".ToCharArray(), cipherTextData);
            Console.WriteLine(string.Format("Decrypt FF3-1 FPE Cipher\tcipherTextData:[{0}]\tLength:[{1}]\tplainTextData:[{2}]\tLength:[{3}]",
                new string(cipherTextData), new string(cipherTextData).Length,
                new string(plainTextData),  new string(plainTextData).Length));            
       }              
  
        Console.WriteLine("==============================");
        Console.WriteLine("End   FPE Cipher Examples.Main");
        Console.WriteLine("==============================");
        Console.WriteLine();
        Console.WriteLine();        
    }
    
    /// <summary>
    /// Generate appropriate random key 
    /// </summary>        
    public static KeyParameter keyParameterGeneration(int keySize)
    {
        CipherKeyGenerator keyGen = new CipherKeyGenerator();        
        keyGen.Init(new KeyGenerationParameters(ExValues.cSharpFixedRandom, keySize));
        KeyParameter keyParam = keyGen.GenerateKeyParameter();
        return keyParam;
    }


    /// <summary>
    /// Example 18 – FF3-1 FPE Mode Encryption
    /// </summary>        
    public static char[] ff3_1FPEEncrypt(KeyParameter keyParam, char[] alphabet, char[] plainTextData)
    {
        // Create a mapper from our alphabet to indexs
        IAlphabetMapper alphabetMapper = new BasicAlphabetMapper(alphabet);

        // Create FpeParameter object
        byte[] tweak = ExValues.sampleTweak;
        FpeParameters fpeKeyParam = new FpeParameters(keyParam, alphabetMapper.Radix, tweak);

        IBlockCipher cipher = new AesEngine();        
        FpeFf3_1Engine cipherMode = new FpeFf3_1Engine(cipher);        
        cipherMode.Init(true,fpeKeyParam);
        
        byte[] cipherTextData = new byte[plainTextData.Length];
        byte[] convertedPlainTextData = alphabetMapper.ConvertToIndexes(plainTextData);
        
        int result = cipherMode.ProcessBlock(convertedPlainTextData, 0, convertedPlainTextData.Length, 
            cipherTextData, 0);

        char[] convertedCipherTextData = alphabetMapper.ConvertToChars(cipherTextData);
        return convertedCipherTextData;
    }

    /// <summary>
    /// Example 18 – FF3-1 FPE Mode Decryption
    /// </summary>        
    public static char[] ff3_1FPEDecrypt(KeyParameter keyParam, char[] alphabet, char[] cipherTextData)
    {
        // Create a mapper from our alphabet to indexs
        IAlphabetMapper alphabetMapper = new BasicAlphabetMapper(alphabet);

        // Create FpeParameter object
        byte[] tweak = ExValues.sampleTweak;
        
        //check what happens when we change false to true below which is the opposite of the encryption
        FpeParameters fpeKeyParam = new FpeParameters(keyParam, alphabetMapper.Radix, tweak);

        IBlockCipher cipher = new AesEngine();        
        FpeFf3_1Engine cipherMode = new FpeFf3_1Engine(cipher);        
        cipherMode.Init(false,fpeKeyParam);
        
        byte[] plainTextData = new byte[cipherTextData.Length];
        byte[] convertedCipherTextData = alphabetMapper.ConvertToIndexes(cipherTextData);
        
        int result = cipherMode.ProcessBlock(convertedCipherTextData, 0, convertedCipherTextData.Length, 
            plainTextData, 0);

        char[] convertedPlainTextData = alphabetMapper.ConvertToChars(plainTextData);
        return convertedPlainTextData;
    }

  
}
